<?php

namespace Drupal\fox\Drush\Commands;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\fox\FoxEntityQuery;
use Drupal\fox\Plugin\FoxCommandsManager;
use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 */
class FoxCommands extends DrushCommands {

  /**
   * Options list.
   *
   * @var array
   */
  protected $options;

  /**
   * Variables list.
   *
   * @var array
   */
  protected $variables = [];

  /**
   * Constructs a FoxCommands object.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly FoxCommandsManager $foxCommandsManager,
    private readonly FoxEntityQuery $foxEntityQuery,
  ) {
    parent::__construct();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.fox_commands'),
      $container->get('fox.query.entity'),
    );
  }

  /**
   * Fox console help.
   */
  #[CLI\Command(name: 'fox:help', aliases: ['foxh'])]
  public function help() {
    $header = [dt('Command'), dt('Description')];
    $data = [];
    $foxCommands = $this->foxCommandsManager->getDefinitions();
    foreach ($foxCommands as $name => $info) {
      $data[$name] = [strtoupper($name), $info['label']];
    }
    ksort($data);

    $this->io()->table($header, $data);

    $header = [dt('Query'), dt('Description')];
    $data = [];
    $data[] = [
      dt('Entity query'),
      dt('[SELECT] [FROM entity_type[.bundle]] [WHERE conditions] [ORDER BY orders] [LIMIT limit OFFSET offset]'),
    ];

    $this->io()->table($header, $data);
  }

  /**
   * Fox console.
   */
  #[CLI\Command(name: 'fox:console', aliases: ['foxc'])]
  #[CLI\Option(name: 'input', description: 'Input commands')]
  #[CLI\Option(name: 'output', description: 'Output commands')]
  #[CLI\Option(name: 'mode', description: 'Command mode: default|debug')]
  public function console(array $options = [
    'input' => '',
    'mode' => 'default',
  ]) {
    $this->options = $options;
    $this->io()->info(dt("Welcome to the Fox console!"));
    $commands_history = [];

    // Enter the interactive loop.
    while (TRUE) {
      if (!empty($options['input'])) {
        $input = trim($options['input']);
        $options['input'] = NULL;
      }
      else {
        $input = $this->io()->ask(dt('Enter a command (type "exit" or "quit" to quit): '));
      }
      $input = strtolower(trim($input));

      // All commands.
      $commands = array_filter(explode(';', $input));
      foreach ($commands as $command) {
        $command = trim($command);

        // Check for exit condition.
        if (in_array($command, ['exit', 'quit'])) {
          $this->io()->info('Exiting the Fox console.');

          // Output commands.
          if (!empty($options['output'])) {
            $header = [
              '#',
              dt('Command'),
            ];
            $output = [];
            foreach ($commands_history as $number => $data) {
              $output[] = [$number + 1, $data];
            }
            $this->io()->table($header, $output);
          }
          return;
        }

        $completed = FALSE;

        // Fox commands.
        if (!$completed) {
          $result = $this->parseFoxCommands($command);
          if (!empty($result)) {
            $this->setVariables($result);
            $success = $this->printResult($result);
            if ($success) {
              $commands_history[] = $command;
            }
            $completed = TRUE;
          }
        }

        // Run Entity Query.
        if (!$completed) {
          $result = $this->foxEntityQuery->doEntityQuery($command, $this->variables, $this->options);
          if (!empty($result)) {
            $this->setVariables($result);
            $success = $this->printResult($result);
            if ($success) {
              $commands_history[] = $command;
            }
            $completed = TRUE;
          }
        }

        // Show variables.
        $message = '';
        foreach ($this->variables as $name => $value) {
          if ($value !== NULL) {
            $message .= "$name=\"$value\" ";
          }
        }
        $this->output->writeln($message);
      }
    }
  }

  /**
   * Fox commands parse.
   *
   * @param string $input
   *   User input.
   *
   * @return string
   *   Output message.
   */
  protected function parseFoxCommands(string $input) {
    $foxCommands = $this->foxCommandsManager->getDefinitions();

    $params = array_filter(explode(' ', $input), function ($value) {
      if (is_numeric($value)) {
        return TRUE;
      }
      return !empty($value);
    });
    $command = strtolower($params[0]);
    if (!empty($foxCommands[$command])) {
      array_shift($params);
      $instance = $this->foxCommandsManager->createInstance($command);
      $result = $instance->execute($params, $this->variables);
      return $result;
    }

    return NULL;
  }

  /**
   * Print result.
   *
   * @param array $result
   *   Command result.
   *
   * @return bool
   *   Success result output.
   */
  protected function printResult(array $result): bool {
    if (empty($result)) {
      return TRUE;
    }

    if (isset($result['error'])) {
      $this->io()->error($result['error']);
      return FALSE;
    }

    if (isset($result['debug'])) {
      $this->io()->info($result['debug']);
    }

    $message = $result['message'] ?? NULL;
    if (!empty($message)) {
      if (is_array($message)) {
        if (empty($message['data'])) {
          $this->io()->warning(dt('Empty data'));
        }
        else {
          if (!empty($message['horizontal'])) {
            $this->io()->horizontalTable($message['header'], $message['data']);
          }
          else {
            $this->io()->table($message['header'], $message['data']);
          }
          $this->output()->writeln(dt('Count: @count', [
            '@count' => count($message['data']),
          ]));
        }
      }
      else {
        $this->output()->writeln($message);
      }

      return TRUE;
    }

  }

  /**
   * Set local variables.
   *
   * @param array $result
   *   Command result.
   */
  protected function setVariables($result): void {
    $variables = $result['variables'] ?? NULL;
    if (!$variables) {
      return;
    }

    foreach ($variables as $name => $value) {
      $this->variables[$name] = $value;
    }
  }

}
